@@ -28,7 +28,7 @@ package RundeckAPI;
 use strict;
 use warnings;
 use POSIX qw(setlocale strftime);
-use File::Basename;			# get basename()
+use File::Basename;
 use LWP::UserAgent;
 use Data::Dumper;
 use HTTP::Cookies;
@@ -38,13 +38,13 @@ use JSON;
 use Storable qw(dclone);
 use Exporter qw(import);
 
-our @EXPORT_OK = qw(get post put delete postFile putFile);
+our @EXPORT_OK = qw(get post put delete postData putData);
 
 #####
 ## CONSTANTS
 #####
 our $TIMEOUT = 10;
-our $VERSION = "1.2.6";
+our $VERSION = "1.3.0";
 #####
 ## VARIABLES
 #####
@@ -55,16 +55,17 @@ our $VERSION = "1.2.6";
 
 sub new {
 	my($class, %args) = @_;
-	my $rc=0;
+	my $rc = 403;
 	my $self = {
 		'url'		=> $args{'url'},
 		'login'		=> $args{'login'},
 		'token'		=> $args{'token'},
-		'password'	=> $args{'password'},
 		'debug'		=> $args{'debug'} || 0,
 		'verbose'	=> $args{'verbose'} || 0,
 		'result'	=> undef,
+		'timeout'	=> $args{'timeout'},
 	};
+	$self->{'timeout'} = $TIMEOUT if not (defined  $args{'timeout'});
 # create and store a cookie jar
 	my $cookie_jar = HTTP::Cookies->new(
 		autosave		=> 1,
@@ -76,11 +77,11 @@ sub new {
 	my($prog, $dirs, $suffix) = fileparse($0, (".pl"));
 	my $ua = LWP::UserAgent->new(
 		'agent'			=> $prog . "-" . $VERSION,
-		'timeout'		=> $TIMEOUT,
+		'timeout'		=> $self->{'timeout'},
 		'cookie_jar'	=> $self->{'cookie_jar'},
 		'requests_redirectable' => ['GET', 'HEAD', 'POST', 'PUT', 'DELETE'],
 		);
-	$ua->show_progress ($args{'debug'});
+	$ua->show_progress ($args{'verbose'});
  	$ua->proxy( ['http', 'https'], $args{'proxy'}) if (defined $args{'proxy'});
 	$self->{'ua'} = $ua;
 
@@ -95,44 +96,36 @@ sub new {
 	$client->addHeader ("Accept", "application/json");
 	$self->{'client'} = $client;
 
-# if we have a token, use it
 	if (defined $self->{'token'}) {
 		$client->addHeader ("X-Rundeck-Auth-Token", $self->{'token'});
 		$client->GET("/api/21/tokens/$self->{'login'}");
-# check if token match id, just to be sure
+# check if token match id
 		my $authJSON = $client->responseContent();
 		$rc = $client->responseCode ();
-		if ($rc-$rc%100 == 200) {
+		if (($rc-$rc%100 == 200) && (index($client->{'_res'}{'_content'}, 'alert alert-danger') == -1)) {
 			my $jHash = decode_json ($authJSON);
 			if (defined $jHash->[0]) {
-				if ($jHash->[0]->{'user'} ne $self->{'login'}) {
-	# should this really happen ?
+				my $connOK = 0;
+				foreach my $tokenInfo (@{$jHash}) {
+					if ($tokenInfo->{'user'} eq $self->{'login'}) {
+						$connOK = 1;
+						last;
+					}
+				}
+				if ($connOK) {
+					$rc = 200;
+				} else {
 					$rc = 403;
 				}
-				else {
-					$rc = 200;
-				}
-			} else {
-	# empty hash, but res = OK
-				$rc = 200;
-			}
-		}
-	} else {
-# post user/passwd
-		$client->POST(
-			"j_security_check",
-			"j_username=$self->{'login'}" . "&" . "j_password=$self->{'password'}",
-		);
-		$rc = $client->responseCode ();
-	}
-	my %hash = ();
 
-## Dirty job, but Rundeck doent follow the REST norm, returning 200 even if auth fails
-## However, this is safe, since if string not found it will fail at get/put/whatever time
-	if (index($client->{'_res'}{'_content'}, 'alert alert-danger') != -1) {
-		$rc = 403;
+			} else {
+				$rc = 403;
+			}
+		} else {
+			$rc = 403;
+		}
 	}
-	if ($rc != 200) {
+	if ($rc-$rc%100 != 200) {
 		$self->{'result'}->{'reqstatus' } = 'UNKN';
 	} else {
 		$self->{'result'}->{'reqstatus'} = 'OK';
@@ -141,7 +134,7 @@ sub new {
 
 # done, bless object and return it
 	bless ($self, $class);
-	$self->_logV1 ("Connected to $self->{'url'}");
+	$self->_logV1 ("Connected to $self->{'url'}") if ($rc-$rc%100 == 200);
 	$self->_logD($self);
 	return $self;
 }
@@ -165,13 +158,9 @@ sub get (){		# endpoint
 		$responsehash->{'reqstatus'} = 'CRIT';
 		$responsehash->{'httpstatus'} = $rc;
 	} else {
+		my $responseType = $self->{'client'}->responseHeader('content-type');
 		my $responseContent = $self->{'client'}->responseContent();
-	# handle special case where test is "ping", response is "pong" in plain text, not a json
-		if ($endpoint =~ /ping/) {
-			$responsehash->{'reqstatus'} = $responseContent =~ /pong/ ? 'OK' : 'CRIT';
-			return dclone ($responsehash);
-		}
-		$responsehash = $self->_handleResponse($rc, $responseContent);
+		$responsehash = $self->_handleResponse($rc, $responseType, $responseContent);
 	}
 	return dclone ($responsehash);
 }
@@ -193,8 +182,9 @@ sub post(){		# endpoint, json
 		$responsehash->{'reqstatus'} = 'CRIT';
 		$responsehash->{'httpstatus'} = $rc;
 	} else {
+		my $responseType = $self->{'client'}->responseHeader('content-type');
 		my $responseContent = $self->{'client'}->responseContent();
-		$responsehash = $self->_handleResponse($rc, $responseContent);
+		$responsehash = $self->_handleResponse($rc, $responseType, $responseContent);
 	}
 	return dclone ($responsehash);
 }
@@ -216,8 +206,9 @@ sub put(){		# endpoint, json
 		$responsehash->{'reqstatus'} = 'CRIT';
 		$responsehash->{'httpstatus'} = $rc;
 	} else {
+		my $responseType = $self->{'client'}->responseHeader('content-type');
 		my $responseContent = $self->{'client'}->responseContent();
-		$responsehash = $self->_handleResponse($rc, $responseContent);
+		$responsehash = $self->_handleResponse($rc, $responseType, $responseContent);
 	}
 	return dclone ($responsehash);
 }
@@ -237,13 +228,23 @@ sub delete () {		# endpoint
 		$responsehash->{'reqstatus'} = 'CRIT';
 		$responsehash->{'httpstatus'} = $rc;
 	} else {
+		my $responseType = $self->{'client'}->responseHeader('content-type');
 		my $responseContent = $self->{'client'}->responseContent();
-		$responsehash = $self->_handleResponse($rc, $responseContent);
+		$responsehash = $self->_handleResponse($rc, $responseType, $responseContent);
 	}
 	return dclone ($responsehash);
 }
 
-sub postFile() {		# endpoint, mimetype, data
+sub postFile() {	# for compatibility only
+	my $self = shift;
+	my $endpoint = shift;
+	my $mimetype = shift;
+	my $data = shift;
+
+	reeturn $self->postData($endpoint, $mimetype, $data);
+
+}
+sub postData() {		# endpoint, mimetype, data
 	my $self = shift;
 	my $endpoint = shift;
 	my $mimetype = shift;
@@ -261,14 +262,23 @@ sub postFile() {		# endpoint, mimetype, data
 		$responsehash->{'reqstatus'} = 'CRIT';
 		$responsehash->{'httpstatus'} = $rc;
 	} else {
+		my $responseType = $self->{'client'}->responseHeader('content-type');
 		my $responseContent = $self->{'client'}->responseContent();
-		$responsehash = $self->_handleResponse($rc, $responseContent);
+		$responsehash = $self->_handleResponse($rc, $responseType, $responseContent);
 	}
 	return dclone ($responsehash);
-
 }
 
-sub putFile() {		# endpoint, mimetype, data
+sub putFile() {	# for compatibility only, must use a ... what ? File path or file handle ?
+	my $self = shift;
+	my $endpoint = shift;
+	my $mimetype = shift;
+	my $data = shift;
+
+	reeturn $self->putData($endpoint, $mimetype, $data);
+
+}
+sub putData() {		# endpoint, mimetype, data
 	my $self = shift;
 	my $endpoint = shift;
 	my $mimetype = shift;
@@ -286,40 +296,51 @@ sub putFile() {		# endpoint, mimetype, data
 		$responsehash->{'reqstatus'} = 'CRIT';
 		$responsehash->{'httpstatus'} = $rc;
 	} else {
+		my $responseType = $self->{'client'}->responseHeader('content-type');
 		my $responseContent = $self->{'client'}->responseContent();
-		$responsehash = $self->_handleResponse($rc, $responseContent);
+		$responsehash = $self->_handleResponse($rc, $responseType, $responseContent);
 	}
 	return dclone ($responsehash);
-
 }
 
 
 sub _handleResponse () {
 	my $self = shift;
 	my $rc = shift;
+	my $responseType = shift;
 	my $responseContent = shift;
 
 	my $responseJSON = ();
 	my $responsehash = ();
-
-	$self->_logV2($responseContent);
-	$responseJSON = decode_json($responseContent) if $responseContent ne '';
-	my $reftype = reftype($responseJSON);
-	if (not defined $reftype) {
-		$self->_bomb("Can't decode undef type");
-	} elsif ($reftype eq 'ARRAY') {
-
-		$responsehash->{'arraycount'} = $#$responseJSON+1;
-		for (my $i = 0; $i <= $#$responseJSON; $i++) {
-			$responsehash->{$i} = $responseJSON->[$i];
-		}
-	} elsif ($reftype eq 'SCALAR') {
-		$self->_bomb("Can't decode scalar type");
-	} elsif ($reftype eq 'HASH') {
-		$responsehash = dclone ($responseJSON);
-	}
 	$responsehash->{'reqstatus'} = 'OK';
-	$responsehash->{'httpstatus'} = $rc;
+	$responsehash->{'httpstatus'} = 200;
+
+	# is data JSON ?
+	if ($responseType eq 'application/json') {
+		$self->_logV2($responseContent);
+		$responseJSON = decode_json($responseContent) if $responseContent ne '';
+		my $reftype = reftype($responseJSON);
+		if (not defined $reftype) {
+			$self->_bomb("Can't decode undef type");
+		} elsif ($reftype eq 'ARRAY') {
+
+			$responsehash->{'arraycount'} = $#$responseJSON+1;
+			for (my $i = 0; $i <= $#$responseJSON; $i++) {
+				$responsehash->{$i} = $responseJSON->[$i];
+			}
+		} elsif ($reftype eq 'SCALAR') {
+			$self->_bomb("Can't decode scalar type");
+		} elsif ($reftype eq 'HASH') {
+			$responsehash = dclone ($responseJSON);
+		}
+		$responsehash->{'reqstatus'} = 'OK';
+		$responsehash->{'httpstatus'} = $rc;
+	} elsif ($responseType eq 'text/plain') {
+		$self->_logV2($responseContent);
+		$responsehash->{'content'} = $responseContent;
+	} else { # assume binary, like text, but do not log
+		$responsehash->{'content'} = $responseContent;
+	}
 	return $responsehash;
 }
 
@@ -359,7 +380,7 @@ sub _bomb() {
 	my $self = shift;
 	my $msg = shift;
 
-	$msg .= "\nReport this to xavier.humbert\@ac-nancy-metz.fr";
+	$msg .= "\nReport this to xavier.humbert\@ac-nancy-metz.fr with detail of the REST call you made";
 	die $msg;
 }
 1;
@@ -377,8 +398,6 @@ RundeckAPI - simplifies authenticate, connect, queries to a Rundeck instance via
 	my $api = RundeckAPI->new(
 		'url'		=> "https://my.rundeck.instance:4440",
 		'login'		=> "admin",
-		'password'	=> "passwd",
-	# OR token, takes precedence
 		'token'		=> <token as generated with GUI, as an admin>
 		'debug'		=> 1,
  		'proxy'		=> "http://proxy.mycompany.com/",
@@ -393,7 +412,8 @@ RundeckAPI - simplifies authenticate, connect, queries to a Rundeck instance via
 
 =item C<new>
 
-Returns an object authenticated and connected to a Rundeck Instance
+Returns an object authenticated and connected to a Rundeck Instance.
+The field 'login' is not stricto sensu required, but it is a good security mesure to check if login/token match
 
 =item C<get>
 
@@ -411,8 +431,27 @@ Sends a PUT query. Similar to post
 
 Sends a DELETE query. Similar to get
 
+=item C<postData>
+
+POST some data. Request three arguments : endpoint, mime-type and the appropriate data. Returns a hash reference.
+
+=item C<putData>
+
+PUT some data. Similar to postData
+
 =back
 
+=head1 RETURN VALUE
+
+Returns a hash reference containing the data sent by Rundeck.
+
+The returned value is structured like the following :
+
+the fields `httpstatus` (200, 403, etc) and `requstatus` (OK, CRIT) are always present.
+
+the content is in the hash itself if the data type is JSON, else in the field `content` if we have text or binary
+
+
 =head1 SEE ALSO
 
 See documentation for Rundeck's API https://docs.rundeck.com/docs/api/rundeck-api.html and returned data
